@@ -1098,6 +1098,7 @@ blaster_free (Display *dpy, Window window, void *closure)
 static const char *blaster_defaults [] = {
   ".background:	black",
   ".foreground:	white",
+  "*fpsSolid:	true",
   "*r_color0:	#FF00FF",
   "*r_color1:	#FFA500",
   "*r_color2:	#FFFF00",
@@ -1128,6 +1129,9 @@ static const char *blaster_defaults [] = {
   "*move_stars_x: 2",
   "*move_stars_y: 1",
   "*move_stars_random: 0",
+#ifdef USE_IPHONE
+  "*ignoreRotation: True",
+#endif
   0
 };
 
